"""
Модуль для распознавания эмоций в тексте
"""
import sys
from typing import List, Union
import torch
import warnings
from transformers import PreTrainedTokenizerBase

from aniemore.utils.classes import (
    BaseRecognizer,
    RecognizerOutputMany,
    RecognizerOutputOne,
)


class TextRecognizer(BaseRecognizer):
    def _get_torch_scores(
            self,
            text: Union[str, List[str]],
            tokenizer: PreTrainedTokenizerBase,
            device: str,
            max_length: int = 512,
            padding: bool = True,
            truncation: bool = True) -> torch.Tensor:
        """[PROTECTED METHOD] Получаем лист текстов, токенизируем, отправляем в модель и возвращаем тензор с вероятностями

        Args:
          text: текст для анализа
          tokenizer: токенайзер
          device: cpu' or 'cuda' or 'cuda:<number>'
          max_length: максимальная длина текста (default=512)
          padding: если True, то добавляем паддинги (default=True)
          truncation: если True, то обрезаем текст (default=True)

        Returns:
          torch.Tensor

        """
        inputs = tokenizer(
            text,
            max_length=max_length,
            padding=padding,
            truncation=truncation,
            return_tensors='pt').to(device)

        with torch.no_grad():
            logits = self._model.to(self.device)(**inputs).logits

        if self._logistic_fct is torch.sigmoid:
            scores = self._logistic_fct(logits)
        elif self._logistic_fct is torch.softmax:
            scores = self._logistic_fct(logits, dim=1)
        else:
            raise ValueError('logistic_fct must be one of torch.sigmoid or torch.softmax')

        return scores

    def _recognize_one(self, text: str) -> RecognizerOutputOne:
        """[PROTECTED METHOD] Получаем строку текста, токенизируем, отправляем в модель и возвращаем "

        Args:
          text(str): текст для анализа

        Returns:
          результат распознования
        """

        scores = self._get_torch_scores(text, self.tokenizer, self.device)

        scores = {k: v for k, v in zip(self.config.id2label.values(), scores[0].tolist())}

        return RecognizerOutputOne(**scores)

    def _recognize_many(self, texts: List[str]) -> RecognizerOutputMany:
        """[PROTECTED METHOD] Принимает список текстов и возвращает список прогнозов.

        Args:
          texts: список текстов для анализа

        Returns:
          результат распознования
        """
        scores = self._get_torch_scores(texts, self.tokenizer, self.device).detach().cpu().numpy()
        results: RecognizerOutputMany = self._get_many_results(texts, scores)

        return results

    def recognize(self, text: Union[List[str], str], return_single_label=False) -> \
            Union[RecognizerOutputOne, RecognizerOutputMany]:

        if self._model is None:
            self._setup_variables()

        if isinstance(text, str):
            if return_single_label:
                return self._get_single_label(self._recognize_one(text))

            return self._recognize_one(text)
        elif isinstance(text, list):
            if return_single_label:
                return self._get_single_label(self._recognize_many(text))

            return self._recognize_many(text)
        else:
            raise ValueError('paths must be str or list')


class TextEnhancer:
    """Класс для улучшения текста, например, для исправления грамматических ошибок и т.д."""
    _grammar_model = None
    _apply_te = None

    def __init__(self, setup_on_init: bool = True) -> None:
        """Инициализация класса
        Args:
          setup_on_init: Если True, модель будет загружена при инициализации класса
        """
        if setup_on_init:
            self._load_model()

    def _load_model(self) -> None:
        """Загрузка модели. Если она уже загружена, то ничего не произойдет."""
        if sys.platform == 'darwin':  # MacOS check
            warning_text = ("Silero models are not supported on MacOS. "
                            "To make it work, we've changed torch engine to `qnnpack`. Use this with caution.")
            warnings.warn(warning_text, category=UserWarning)
            torch.backends.quantized.engine = 'qnnpack'
        if self._grammar_model is None and self._apply_te is None:
            self._grammar_model, _, _, _, self._apply_te = torch.hub.load(repo_or_dir='snakers4/silero-models',
                                                                          model='silero_te')

    def enhance(self, text: str) -> str:
        """Улучшение текста (исправление грамматических ошибок и т.д.)

        Args:
          text: Текст, который нужно улучшить

        Returns:
          Улучшенный текст
        """
        return self._apply_te(text.lower(), lan='ru')



"""Здесь хранятся все модели, используемые в проекте
"""
import dataclasses
from enum import Enum
from typing import NamedTuple, Type
from transformers import (
    Wav2Vec2ForSequenceClassification,
    WavLMForSequenceClassification,
    UniSpeechSatForSequenceClassification,
    HubertForSequenceClassification,
    AutoModelForSequenceClassification,
    BertForSequenceClassification,
    PreTrainedModel
)

from aniemore.custom.models import (
    Wav2Vec2BertForSequenceClassification,
    WavLMBertForSequenceClassification
)


class Model(NamedTuple):
    """
    NamedTuple класс, используемый для более удобного доступа к информации о модели

    Attributes:
     model_cls: HuggingFace класс модели
     model_url: ссылка на модель на HuggingFace

    Examples:
     >>> from aniemore.models import Model
     >>> my_model: Model = Model(BertForSequenceClassification, 'user/model-repo-link')
     >>> # load `my_model` to recognizer class
    """
    model_cls: Type[PreTrainedModel]
    model_url: str


@dataclasses.dataclass(frozen=True)
class HuggingFaceModel:
    """
    References:
        Our models and datasets placed here: https://huggingface.co/Aniemore
    """
    class Voice(Model, Enum):
        """
            Attributes:
                 Wav2Vec2: '`niemore/wav2vec2-emotion-russian-resd`
                 Wav2Vec2_Custom: `aniemore/wav2vec2-xlsr-53-russian-emotion-recognition`
                 WavLM: `aniemore/wavlm-emotion-russian-resd`
                 Hubert: `aniemore/hubert-emotion-russian-resd`
                 UniSpeech: `aniemore/unispeech-sat-emotion-russian-resd`
            References:
                Our models and datasets placed here: https://huggingface.co/Aniemore
        """
        Wav2Vec2 = Model(Wav2Vec2ForSequenceClassification, 'aniemore/wav2vec2-emotion-russian-resd')
        Wav2Vec2_Custom = Model(
            AutoModelForSequenceClassification,
            'aniemore/wav2vec2-xlsr-53-russian-emotion-recognition'
        )
        WavLM = Model(WavLMForSequenceClassification, 'aniemore/wavlm-emotion-russian-resd')
        Hubert = Model(HubertForSequenceClassification, 'aniemore/hubert-emotion-russian-resd')
        UniSpeech = Model(UniSpeechSatForSequenceClassification, 'aniemore/unispeech-sat-emotion-russian-resd')

    class Text(Model, Enum):
        """
            Attributes:
                Bert_Tiny: `aniemore/rubert-tiny2-russian-emotion-detection`
                Bert_Tiny2: `aniemore/rubert-tiny-emotion-russian-cedr-m7`
                Bert_Base: `aniemore/rubert-base-emotion-russian-cedr-m7`
                Bert_Large: `aniemore/rubert-large-emotion-russian-cedr-m7`
            References:
                Our models and datasets placed here: https://huggingface.co/Aniemore
        """
        Bert_Tiny2 = Model(BertForSequenceClassification, 'aniemore/rubert-tiny2-russian-emotion-detection')
        Bert_Tiny = Model(BertForSequenceClassification, 'aniemore/rubert-tiny-emotion-russian-cedr-m7')
        Bert_Base = Model(BertForSequenceClassification, 'aniemore/rubert-base-emotion-russian-cedr-m7')
        Bert_Large = Model(BertForSequenceClassification, 'aniemore/rubert-large-emotion-russian-cedr-m7')

    class MultiModal(Model, Enum):
        """
            Attributes:
                Wav2Vec2BertTiny: `aniemore/wav2vec2-bert-tiny2-emotion-russian-resd`
                Wav2Vec2BertBase: `aniemore/wav2vec2-bert-base-emotion-russian-resd`
                WavLMBertTiny: `aniemore/wavlm-bert-tiny2-emotion-russian-resd`
                WavLMBertBase: `Ar4ikov/wavlm-bert-base-multimodal-emotion-russian-resd`
            References:
                Our models and datasets placed here: https://huggingface.co/Aniemore
        """
        Wav2Vec2BertTiny = Model(
            Wav2Vec2BertForSequenceClassification, 'aniemore/wav2vec2-bert-tiny2-s-emotion-russian-resd')
        Wav2Vec2BertBase = Model(
            Wav2Vec2BertForSequenceClassification, 'aniemore/wav2vec2-bert-base-s-emotion-russian-resd')
        WavLMBertTiny = Model(
            WavLMBertForSequenceClassification, 'aniemore/wavlm-bert-tiny2-s-emotion-russian-resd')
        WavLMBertBase = Model(
            WavLMBertForSequenceClassification, 'aniemore/wavlm-bert-base-s-emotion-russian-resd')



"""Base model classes
"""
from dataclasses import dataclass
from typing import Union, Type

import torch
from transformers.utils import ModelOutput
from transformers import (
    Wav2Vec2ForSequenceClassification,
    WavLMForSequenceClassification,
    UniSpeechSatForSequenceClassification,
    HubertForSequenceClassification,
    PreTrainedModel,
    PretrainedConfig,
    WavLMConfig,
    BertConfig,
    WavLMModel,
    BertModel,
    Wav2Vec2Config,
    Wav2Vec2Model
)


@dataclass
class SpeechModelOutput(ModelOutput):
    """Base class for model's outputs, with potential hidden states and attentions.

    Args:
        loss (:obj:`torch.FloatTensor` of shape :obj:`(1,)`, `optional`, returned when :obj:`labels` is provided):
            Classification (or regression if config.num_labels==1) loss.
        logits (:obj:`torch.FloatTensor` of shape :obj:`(batch_size, config.num_labels)`):
            Classification (or regression if config.num_labels==1) scores (before SoftMax).
        hidden_states (:obj:`tuple(torch.FloatTensor)`, `optional`, returned when ``config.output_hidden_states=True``):
            Tuple of :obj:`torch.FloatTensor` (one for the output of the embeddings + one for the output of each layer)
            of shape :obj:`(batch_size, sequence_length, hidden_size)`. Hidden-states of the model at the output of
            each layer plus the initial embedding outputs.
        attentions (:obj:`tuple(torch.FloatTensor)`, `optional`, returned when ``config.output_attentions=True``):
            Tuple of :obj:`torch.FloatTensor` (one for each layer) of shape :obj:`(batch_size, num_heads,
            sequence_length, sequence_length)`. Attentions weights after the attention softmax, used to compute the
            weighted average in the self-attention heads.

    Examples::
        >>> from transformers import Wav2Vec2ForSequenceClassification, Wav2Vec2Tokenizer
        >>> import torch
        >>>
        >>> tokenizer = Wav2Vec2Tokenizer.from_pretrained("facebook/wav2vec2-base-960h")
        >>> model = Wav2Vec2ForSequenceClassification.from_pretrained("facebook/wav2vec2-base-960h")
        >>> input_values = tokenizer("Hello, my dog is cute", return_tensors="pt").input_values  # Batch size 1
        >>> logits = model(input_values).logits
        >>> assert logits.shape == (1, 2)
    """
    loss: torch.FloatTensor
    logits: torch.FloatTensor = None
    hidden_states: torch.FloatTensor = None
    attentions: torch.FloatTensor = None


class MultiModalConfig(PretrainedConfig):
    """Base class for multimodal configs"""
    def __init__(self, **kwargs):
        super().__init__(**kwargs)


class BaseClassificationModel(PreTrainedModel):  # noqa
    config: Type[Union[PretrainedConfig, None]] = None

    def compute_loss(self, logits, labels):
        """Compute loss

        Args:
            logits (torch.FloatTensor): logits
            labels (torch.LongTensor): labels

        Returns:
            torch.FloatTensor: loss

        Raises:
            ValueError: Invalid number of labels
        """
        if self.config.problem_type is None:
            if self.num_labels == 1:
                self.config.problem_type = "regression"
            elif self.num_labels > 1:
                self.config.problem_type = "single_label_classification"
            else:
                raise ValueError("Invalid number of labels: {}".format(self.num_labels))

        if self.config.problem_type == "single_label_classification":
            loss_fct = torch.nn.CrossEntropyLoss()
            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1))

        elif self.config.problem_type == "multi_label_classification":
            loss_fct = torch.nn.BCEWithLogitsLoss()
            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1, self.num_labels))

        elif self.config.problem_type == "regression":
            loss_fct = torch.nn.MSELoss()
            loss = loss_fct(logits.view(-1), labels.view(-1))
        else:
            raise ValueError("Problem_type {} not supported".format(self.config.problem_type))

        return loss

    @staticmethod
    def merged_strategy(
            hidden_states,
            mode="mean"
    ):
        """Merged strategy for pooling

        Args:
            hidden_states (torch.FloatTensor): hidden states
            mode (str, optional): pooling mode. Defaults to "mean".

        Returns:
            torch.FloatTensor: pooled hidden states
        """
        if mode == "mean":
            outputs = torch.mean(hidden_states, dim=1)
        elif mode == "sum":
            outputs = torch.sum(hidden_states, dim=1)
        elif mode == "max":
            outputs = torch.max(hidden_states, dim=1)[0]
        else:
            raise Exception(
                "The pooling method hasn't been defined! Your pooling mode must be one of these ['mean', 'sum', 'max']")

        return outputs


class BaseModelForVoiceBaseClassification(BaseClassificationModel):  # noqa
    def __init__(self, config, num_labels):
        """Base model for voice classification

        Args:
            config (PretrainedConfig): config
            num_labels (int): number of labels
        """
        super().__init__(config=config)
        self.num_labels = num_labels
        self.pooling_mode = config.pooling_mode
        self.projector = torch.nn.Linear(config.hidden_size, config.classifier_proj_size)
        self.classifier = torch.nn.Linear(config.classifier_proj_size, config.num_labels)

    def forward(
            self,
            input_values,
            attention_mask=None,
            output_attentions=None,
            output_hidden_states=None,
            return_dict=None,
            labels=None,
    ):
        """Forward

        Args:
            input_values (torch.FloatTensor): input values
            attention_mask (torch.LongTensor, optional): attention mask. Defaults to None.
            output_attentions (bool, optional): output attentions. Defaults to None.
            output_hidden_states (bool, optional): output hidden states. Defaults to None.
            return_dict (bool, optional): return dict. Defaults to None.
            labels (torch.LongTensor, optional): labels. Defaults to None.

        Returns:
            torch.FloatTensor: logits

        Raises:
            ValueError: Invalid number of labels
        """
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict
        outputs = self.wavlm(
            input_values,
            attention_mask=attention_mask,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )
        hidden_states = self.projector(outputs.last_hidden_state)
        hidden_states = self.merged_strategy(hidden_states, mode=self.pooling_mode)
        logits = self.classifier(hidden_states)

        loss = None
        if labels is not None:
            loss = self.compute_loss(logits, labels)

        if not return_dict:
            output = (logits,) + outputs[2:]
            return ((loss,) + output) if loss is not None else output

        return SpeechModelOutput(
            loss=loss,
            logits=logits,
            hidden_states=outputs.hidden_states,
            attentions=outputs.attentions,
        )


class BaseMultiModalForSequenceBaseClassification(BaseClassificationModel):  # noqa
    config_class = MultiModalConfig

    def __init__(self, config):
        """
        Args:
            config (MultiModalConfig): config

        Attributes:
            config (MultiModalConfig): config
            num_labels (int): number of labels
            audio_config (Union[PretrainedConfig, None]): audio config
            text_config (Union[PretrainedConfig, None]): text config
            audio_model (Union[PreTrainedModel, None]): audio model
            text_model (Union[PreTrainedModel, None]): text model
            classifier (Union[torch.nn.Linear, None]): classifier
        """
        super().__init__(config)
        self.config = config
        self.num_labels = self.config.num_labels
        self.audio_config: Union[PretrainedConfig, None] = None
        self.text_config: Union[PretrainedConfig, None] = None
        self.audio_model: Union[PreTrainedModel, None] = None
        self.text_model: Union[PreTrainedModel, None] = None
        self.classifier: Union[torch.nn.Linear, None] = None

    def forward(
            self,
            input_ids=None,
            input_values=None,
            text_attention_mask=None,
            audio_attention_mask=None,
            token_type_ids=None,
            position_ids=None,
            head_mask=None,
            inputs_embeds=None,
            labels=None,
            output_attentions=None,
            output_hidden_states=None,
            return_dict=True,
    ):
        """Forward method for multimodal model for sequence classification task (e.g. text + audio)

        Args:
            input_ids (torch.LongTensor, optional): input ids. Defaults to None.
            input_values (torch.FloatTensor, optional): input values. Defaults to None.
            text_attention_mask (torch.LongTensor, optional): text attention mask. Defaults to None.
            audio_attention_mask (torch.LongTensor, optional): audio attention mask. Defaults to None.
            token_type_ids (torch.LongTensor, optional): token type ids. Defaults to None.
            position_ids (torch.LongTensor, optional): position ids. Defaults to None.
            head_mask (torch.FloatTensor, optional): head mask. Defaults to None.
            inputs_embeds (torch.FloatTensor, optional): inputs embeds. Defaults to None.
            labels (torch.LongTensor, optional): labels. Defaults to None.
            output_attentions (bool, optional): output attentions. Defaults to None.
            output_hidden_states (bool, optional): output hidden states. Defaults to None.
            return_dict (bool, optional): return dict. Defaults to True.

        Returns:
            torch.FloatTensor: logits
        """
        audio_output = self.audio_model(
            input_values=input_values,
            attention_mask=audio_attention_mask,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict
        )
        text_output = self.text_model(
            input_ids=input_ids,
            attention_mask=text_attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
            head_mask=head_mask,
            inputs_embeds=inputs_embeds,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )
        audio_mean = self.merged_strategy(audio_output.last_hidden_state, mode="mean")

        pooled_output = torch.cat(
            (audio_mean, text_output.pooler_output), dim=1
        )
        logits = self.classifier(pooled_output)
        loss = None

        if labels is not None:
            loss = self.compute_loss(logits, labels)

        return SpeechModelOutput(
            loss=loss,
            logits=logits
        )


class Wav2Vec2ForVoiceClassification(BaseModelForVoiceBaseClassification):  # noqa
    """
    Wav2Vec2ForVoiceClassification is a model for voice classification task
     (e.g. speech command, voice activity detection, etc.)

    Args:
        config (Wav2Vec2Config): config
        num_labels (int): number of labels

    Attributes:
        config (Wav2Vec2Config): config
        num_labels (int): number of labels
        wav2vec2 (Wav2Vec2ForSequenceClassification): wav2vec2 model
    """
    def __init__(self, config, num_labels):
        super().__init__(config, num_labels)
        self.wav2vec2 = Wav2Vec2ForSequenceClassification(config)
        self.init_weights()


class WavLMForVoiceClassification(BaseModelForVoiceBaseClassification):  # noqa
    """
    WavLMForVoiceClassification is a model for voice classification task
     (e.g. speech command, voice activity detection, etc.)

    Args:
        config (WavLMConfig): config
        num_labels (int): number of labels

    Attributes:
        config (WavLMConfig): config
        num_labels (int): number of labels
        wavlm (WavLMForSequenceClassification): wavlm model
    """
    def __init__(self, config, num_labels):
        super().__init__(config, num_labels)
        self.wavlm = WavLMForSequenceClassification(config)
        self.init_weights()


class UniSpeechSatForVoiceClassification(BaseModelForVoiceBaseClassification):  # noqa
    """
    UniSpeechSatForVoiceClassification is a model for voice classification task
     (e.g. speech command, voice activity detection, etc.)

    Args:
        config (UniSpeechSatConfig): config
        num_labels (int): number of labels

    Attributes:
        config (UniSpeechSatConfig): config
        num_labels (int): number of labels
        unispeech_sat (UniSpeechSatForSequenceClassification): unispeech_sat model
    """
    def __init__(self, config, num_labels):
        super().__init__(config, num_labels)
        self.unispeech_sat = UniSpeechSatForSequenceClassification(config)
        self.init_weights()


class HubertForVoiceClassification(BaseModelForVoiceBaseClassification):  # noqa
    """
    HubertForVoiceClassification is a model for voice classification task
     (e.g. speech command, voice activity detection, etc.)

    Args:
        config (HubertConfig): config
        num_labels (int): number of labels

    Attributes:
        config (HubertConfig): config
        num_labels (int): number of labels
        hubert (HubertForSequenceClassification): hubert model
    """
    def __init__(self, config, num_labels):
        super().__init__(config, num_labels)
        self.hubert = HubertForSequenceClassification(config)
        self.init_weights()


class Wav2Vec2BertForSequenceClassification(BaseMultiModalForSequenceBaseClassification):  # noqa
    """
    Wav2Vec2BertForSequenceClassification is a model for sequence classification task
     (e.g. sentiment analysis, text classification, etc.)

    Args:
        config (Wav2Vec2BertConfig): config

    Attributes:
        config (Wav2Vec2BertConfig): config
        audio_config (Wav2Vec2Config): wav2vec2 config
        text_config (BertConfig): bert config
        audio_model (Wav2Vec2Model): wav2vec2 model
        text_model (BertModel): bert model
        classifier (torch.nn.Linear): classifier
    """
    def __init__(self, config):
        super().__init__(config)
        self.audio_config = Wav2Vec2Config.from_dict(self.config.Wav2Vec2Model)
        self.text_config = BertConfig.from_dict(self.config.BertModel)
        self.audio_model = Wav2Vec2Model(self.audio_config)
        self.text_model = BertModel(self.text_config)
        self.classifier = torch.nn.Linear(
            self.audio_config.hidden_size + self.text_config.hidden_size, self.num_labels
        )
        self.init_weights()


class WavLMBertForSequenceClassification(BaseMultiModalForSequenceBaseClassification):  # noqa
    """
    WavLMBertForSequenceClassification is a model for sequence classification task
     (e.g. sentiment analysis, text classification, etc.)

    Args:
        config (WavLMBertConfig): config

    Attributes:
        config (WavLMBertConfig): config
        audio_config (WavLMConfig): wavlm config
        text_config (BertConfig): bert config
        audio_model (WavLMModel): wavlm model
        text_model (BertModel): bert model
        classifier (torch.nn.Linear): classifier
    """
    def __init__(self, config):
        super().__init__(config)
        self.audio_config = WavLMConfig.from_dict(self.config.WavLMModel)
        self.text_config = BertConfig.from_dict(self.config.BertModel)
        self.audio_model = WavLMModel(self.audio_config)
        self.text_model = BertModel(self.text_config)
        self.classifier = torch.nn.Linear(
            self.audio_config.hidden_size + self.text_config.hidden_size, self.num_labels
        )
        self.init_weights()


class FineTuneWav2Vec2BertForSequenceClassification(BaseMultiModalForSequenceBaseClassification):  # noqa
    """
    FineTuneWav2Vec2BertForSequenceClassification is a model for sequence classification task
     (e.g. sentiment analysis, text classification, etc.) for fine-tuning

    Args:
        config (Wav2Vec2BertConfig): config

    Attributes:
        config (Wav2Vec2BertConfig): config
        audio_config (Wav2Vec2Config): wav2vec2 config
        text_config (BertConfig): bert config
        audio_model (Wav2Vec2Model): wav2vec2 model
        text_model (BertModel): bert model
        classifier (torch.nn.Linear): classifier
    """
    def __init__(self, config):
        super().__init__(config)
        self.audio_config = Wav2Vec2Config.from_dict(self.config.Wav2Vec2Model)
        self.text_config = BertConfig.from_dict(self.config.BertModel)
        self.audio_model = Wav2Vec2Model.from_pretrained(self.audio_config._name_or_path, config=self.audio_config)
        self.text_model = BertModel.from_pretrained(self.text_config._name_or_path, config=self.text_config)
        self.classifier = torch.nn.Linear(
            self.audio_config.hidden_size + self.text_config.hidden_size, self.num_labels
        )
        self.init_weights()


class FineTuneWavLMBertForSequenceClassification(BaseMultiModalForSequenceBaseClassification):  # noqa
    """
    FineTuneWavLMBertForSequenceClassification is a model for sequence classification task
     (e.g. sentiment analysis, text classification, etc.) for fine-tuning

    Args:
        config (WavLMBertConfig): config

    Attributes:
        config (WavLMBertConfig): config
        audio_config (WavLMConfig): wavlm config
        text_config (BertConfig): bert config
        audio_model (WavLMModel): wavlm model
        text_model (BertModel): bert model
        classifier (torch.nn.Linear): classifier
    """
    def __init__(self, config):
        super().__init__(config)
        self.audio_config = WavLMConfig.from_dict(self.config.WavLMModel)
        self.text_config = BertConfig.from_dict(self.config.BertModel)
        self.audio_model = WavLMModel.from_pretrained(self.audio_config._name_or_path, config=self.audio_config)
        self.text_model = BertModel.from_pretrained(self.text_config._name_or_path, config=self.text_config)
        self.classifier = torch.nn.Linear(
            self.audio_config.hidden_size + self.text_config.hidden_size, self.num_labels
        )
        self.init_weights()



from typing import Union, List
import numpy
import torch
import torchaudio

from aniemore.utils.classes import (
    BaseRecognizer,
    RecognizerOutputOne,
    RecognizerOutputMany,
)


class VoiceRecognizer(BaseRecognizer):
    @classmethod
    def speech_file_to_array_fn(cls, path):
        """Загружаем аудиофайл в массив

        Args:
          path: путь к файлу

        Returns:
          numpy.ndarray

        """
        speech_array, _sampling_rate = torchaudio.load(path)
        resampler = torchaudio.transforms.Resample(_sampling_rate)
        speech = resampler(speech_array).squeeze().numpy()
        return speech

    def _get_torch_scores(self, speech: Union[List[torch.Tensor], torch.Tensor, numpy.ndarray]) -> torch.Tensor:
        """Получаем выход модели

        Args:
          speech: ndarray

        Returns:
          torch.Tensor

        """
        sampling_rate = self.feature_extractor.sampling_rate
        inputs = self.feature_extractor(speech, sampling_rate=sampling_rate, return_tensors="pt", padding=True)
        inputs = {key: inputs[key].to(self._model.device) for key in inputs}

        with torch.no_grad():
            logits = self._model(**inputs).logits

        # move inputs to cpu back
        for key in inputs:
            inputs[key] = inputs[key].cpu()

        del inputs

        if self._logistic_fct is torch.sigmoid:
            scores = self._logistic_fct(logits)
        elif self._logistic_fct is torch.softmax:
            scores = self._logistic_fct(logits, dim=1)
        else:
            raise ValueError('logistic_fct must be torch.sigmoid or torch.softmax')

        return scores

    def _recognize_one(self, path: str) -> RecognizerOutputOne:
        """Исполнение рантайма для одного файла

        Args:
          path: путь к файлу

        Returns:
          RecognizerOutputOne

        """
        speech = self.speech_file_to_array_fn(path)
        scores = self._get_torch_scores(speech)

        scores = {k: v for k, v in zip(self.config.id2label.values(), scores[0].tolist())}

        return RecognizerOutputOne(**scores)

    def _recognize_many(self, paths: List[str]) -> RecognizerOutputMany:
        """Прогнозируем несколько файлов

        Args:
          paths: список путей к файлам

        Returns:
          словарь с выходами модели

        """
        speeches = []
        for path in paths:
            speech = self.speech_file_to_array_fn(path)
            speeches.append(speech)
        scores: torch.Tensor = self._get_torch_scores(speeches)
        results: RecognizerOutputMany = self._get_many_results(paths, scores)

        return results

    def recognize(
            self,
            paths: Union[List[str], str],
            return_single_label: bool = False
    ) -> Union[RecognizerOutputOne, RecognizerOutputMany]:
        """Прогнозируем файлы

        Args:
          paths: путь к файлу или список путей к файлам
          return_single_label: если True, то возвращаем только один лейбл

        Returns:
          выход модели

        """
        if self._model is None:
            self._setup_variables()

        if isinstance(paths, str):
            if return_single_label:
                return self._get_single_label(self._recognize_one(paths))
            return self._recognize_one(paths)

        elif isinstance(paths, list):
            if return_single_label:
                return self._get_single_label(self._recognize_many(paths))
            return self._recognize_many(paths)

        else:
            raise ValueError('paths must be str or list')



from enum import Enum
from typing import NamedTuple


class Dataset(NamedTuple):
    dataset_name: str
    dataset_url: str


class HuggingFaceDataset(Dataset, Enum):
    RESD = Dataset('Russian Emotional Speech Dialoges', 'aniemore/resd')
    RESD_ANNOTATED = Dataset('Russian Emotional Speech Dialoges [Annotated]', 'aniemore/resd-annotated')
    RESD_ANNOTATED_MULTI = Dataset(
        'Russian Emotional Speech Dialoges [Annotated with multi labels]', 'aniemore/resd-annotated-multi')
    CEDR_M7 = Dataset('Corpus for Emotions Detecting moods 7', 'aniemore/cedr-m7')
